/*! \file int2.c \brief Int2-Haendler */
//***************************************************************************
//*            int2.c
//*
//*  Tue Aug 15 19:59:38 2006
//*  Copyright  2006  sharandac
//*  Email sharandac[at]snafu.de
//****************************************************************************/
///	\ingroup hardware
///	\defgroup INT2 Funktionen fuer dn INT2 (int2.c)
///	\code #include "int2.h" \endcode
///	\par Uebersicht
///		Funktionen um mit den INT2 zu arbeiten.
//****************************************************************************/
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
//@{ 
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include "int2.h"
#include "../hardware/uart.h"

unsigned char interrupt;

INT2_CALLBACK_FUNC INT2_CallbackFunc[ MAX_INT2_CALLBACKS ];

/* -----------------------------------------------------------------------------------------------------------*/
/*! Die Init fuer die INT2.
 */
/* -----------------------------------------------------------------------------------------------------------*/
void INT2_init(void)
	{
		cli();
		
		unsigned char i;
		// Alle Callbackeinträge löschen
		for ( i = 0 ; i < MAX_INT2_CALLBACKS ; i++ ) INT2_CallbackFunc[i] = NULL;
		// Interruptfreigabe für INT2
		GICR |= 0x20;
		
		sei();

	}
			
		

ISR(INT2_vect)       /* veraltet: SIGNAL(SIG_INTERRUPT0) */
	{
		unsigned char i;
		for ( i = 0 ; i < MAX_INT2_CALLBACKS ; i++ ) if ( INT2_CallbackFunc[i] != NULL ) INT2_CallbackFunc[i]();
	}
	
void INT2_disable(void)
	{
		GICR &= 0xdf;
	}
	
void INT2_enable(void)
	{
		GICR |= 0x20;
	}

unsigned char INT2_RegisterCallbackFunction( INT2_CALLBACK_FUNC pFunc )
	{
		unsigned char i;
		
		for ( i = 0 ; i < MAX_INT2_CALLBACKS ; i++ ) 
		{
			if ( INT2_CallbackFunc[i] == pFunc )
				return TRUE;
		}
		
		for ( i = 0 ; i < MAX_INT2_CALLBACKS ; i++ )
		{
			if ( INT2_CallbackFunc[i] == NULL )
			{
				INT2_CallbackFunc[i] = pFunc;
				return TRUE;
			}
		}
		return FALSE;
	}

unsigned char INT2_RemoveCallbackFunction( INT2_CALLBACK_FUNC pFunc )
	{
		unsigned char i;
		
		for ( i = 0 ; i < MAX_INT2_CALLBACKS ; i++ )
		{
			if ( INT2_CallbackFunc[i] == pFunc )
			{
				INT2_CallbackFunc[i] = NULL;
				return TRUE;
			}
		}
		return FALSE;
	}
//}@
