/*! \file "timer1.c" \brief Timer 1 als Zeitbasis benutzen */
//***************************************************************************
//*            timer.c
//*
//*  Sun Jul 16 12:52:34 2006
//*  Copyright  2006  sharandac
//*  Email sharandac(at)snafu.de
//****************************************************************************/
///	\ingroup hardware
///	\defgroup TIMER1 Timer 1 als Zeitbasis benutzen (timer1.c)
///	\code #include "timer1.h" \endcode
///	\par Uebersicht
///		Timer 1 als Zeitbasis fuer igene Programme benutzen. Es koennen damit Timeout
/// Counter gebaut werden, oder beim Funktionen mit in den Timerinterrupt enigebunden werden.
//****************************************************************************/
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
//@{
#include <avr/interrupt.h>
#include <avr/io.h>
#include "timer1.h"

unsigned char ms;
unsigned char ss;
unsigned char mm;
unsigned char hh;

TIMER1_CALLBACK_FUNC TIMER1_sec_CallbackFunc[ MAX_TIMER1_CALLBACKS ];
TIMER1_CALLBACK_FUNC TIMER1_msec_CallbackFunc[ MAX_TIMER1_CALLBACKS ];

unsigned int TIMER1_msec_CountdownTimers[ MAX_TIMER1_COUNTDOWNTIMER ];

/*! \brief Die Timer1 init Funktione
 */
void timer1_init(void)
	{
		unsigned char i;
		
		cli();
		
		// Alle Callbackeinträge löschen
		for ( i = 0 ; i < MAX_TIMER1_CALLBACKS ; i++ ) TIMER1_sec_CallbackFunc[i] = NULL;
		for ( i = 0 ; i < MAX_TIMER1_CALLBACKS ; i++ ) TIMER1_msec_CallbackFunc[i] = NULL;
			
		for ( i = 0 ; i < MAX_TIMER1_COUNTDOWNTIMER ; i++ ) TIMER1_msec_CountdownTimers[i] = COUNDOWNTIMER_UNUSE ;
			
		// Timer1 einstellungen setzen
		TIMSK |= ( 1<<TOIE1 );
		TCCR1B |= ( 1<<CS12 ) | ( 0<<CS11 ) | ( 1<<CS10 );
		TCNT1 = 65535 - ( ( 8000000 / 1024 ) / SECDIV ) ;
//		TCNT1 = 65535 - ( ( F_CPU / 1024 ) / SECDIV ) ;
		
		sei();
	}
	
/*******************************************************************************
 Die ISR für Timer1
*******************************************************************************/
ISR( TIMER1_OVF_vect )
	{
		unsigned char tmp_sreg;  	// temporaerer Speicher fuer das Statusregister
    	tmp_sreg = SREG;   			// Statusregister (also auch das I-Flag darin) sichern
   		cli();             			// Interrupts global deaktivieren

    	unsigned char i;
		
		TCNT1 = 65535 - ( ( 8000000 / 1024 ) / SECDIV ) ;
//		TCNT1 = 65535 - ( ( F_CPU / 1024 ) / SECDIV ) ;

		ms++;

		for ( i = 0 ; i < MAX_TIMER1_CALLBACKS ; i++ ) if ( TIMER1_msec_CallbackFunc[i] != NULL ) TIMER1_msec_CallbackFunc[i]();
		
		// CounterTimer bearbeiten
		for ( i = 0 ; i < MAX_TIMER1_COUNTDOWNTIMER ; i++ )
			if ( TIMER1_msec_CountdownTimers[i] != COUNDOWNTIMER_UNUSE && TIMER1_msec_CountdownTimers[i] != 0 ) 
				TIMER1_msec_CountdownTimers[i]--;
				
		if ( ms == SECDIV )
		{
			ms = 0;
			ss++;

			// CallbackFuinction aufrufen
			for ( i = 0 ; i < MAX_TIMER1_CALLBACKS ; i++ ) if ( TIMER1_sec_CallbackFunc[i] != NULL ) TIMER1_sec_CallbackFunc[i]();
			
			if ( ss == 60 )
			{
				ss = 0;
				mm++;
				if ( mm == 60 )
				{
					mm = 0;
					hh++;
					if ( hh == 24 )
					{
						hh = 0;
					}
				}
			}
		}
		
		SREG = tmp_sreg;    		// Status-Register wieder herstellen 
                      				// somit auch das I-Flag auf gesicherten Zustand setzen
	}

	
/*******************************************************************************
 Fügt eine Callbackfunktion für sekunden hinzu
*******************************************************************************/
unsigned char timer1_sec_RegisterCallbackFunction( TIMER1_CALLBACK_FUNC pFunc )
	{
		unsigned char i;
		
		for ( i = 0 ; i < MAX_TIMER1_CALLBACKS ; i++ ) 
		{
			if ( TIMER1_sec_CallbackFunc[i] == pFunc )
				return TRUE;
		}
		
		for ( i = 0 ; i < MAX_TIMER1_CALLBACKS ; i++ )
		{
			if ( TIMER1_sec_CallbackFunc[i] == NULL )
			{
				TIMER1_sec_CallbackFunc[i] = pFunc;
				return TRUE;
			}
		}
		return FALSE;
	}

/*******************************************************************************
 Enfernt die Callbackfunktion aus der Tabelle für sekunden
*******************************************************************************/
unsigned char timer1_sec_RemoveCallbackFunction( TIMER1_CALLBACK_FUNC pFunc )
	{
		unsigned char i;
		
		for ( i = 0 ; i < MAX_TIMER1_CALLBACKS ; i++ )
		{
			if ( TIMER1_sec_CallbackFunc[i] == pFunc )
			{
				TIMER1_sec_CallbackFunc[i] = NULL;
				return TRUE;
			}
		}
		return FALSE;
	}

/*******************************************************************************
 Fügt eine Callbackfunktion für millisekunden hinzu
*******************************************************************************/
unsigned char timer1_msec_RegisterCallbackFunction( TIMER1_CALLBACK_FUNC pFunc )
	{
		unsigned char i;
		
		for ( i = 0 ; i < MAX_TIMER1_CALLBACKS ; i++ ) 
		{
			if ( TIMER1_msec_CallbackFunc[i] == pFunc )
				return TRUE;
		}
		
		for ( i = 0 ; i < MAX_TIMER1_CALLBACKS ; i++ )
		{
			if ( TIMER1_msec_CallbackFunc[i] == NULL )
			{
				TIMER1_msec_CallbackFunc[i] = pFunc;
				return TRUE;
			}
		}
		return FALSE;
	}

/*******************************************************************************
 Enfernt die Callbackfunktion aus der Tabelle für millisekunden
*******************************************************************************/
unsigned char timer1_msec_RemoveCallbackFunction( TIMER1_CALLBACK_FUNC pFunc )
	{
		unsigned char i;
		
		for ( i = 0 ; i < MAX_TIMER1_CALLBACKS ; i++ )
		{
			if ( TIMER1_msec_CallbackFunc[i] == pFunc )
			{
				TIMER1_msec_CallbackFunc[i] = NULL;
				return TRUE;
			}
		}
		return FALSE;
	}

/*******************************************************************************
 Registriert einen CountdownTimer
*******************************************************************************/
unsigned char timer1_msec_RegisterCoundowntimer( void )
	{
		unsigned char i;
		
		for ( i = 0 ; i < MAX_TIMER1_COUNTDOWNTIMER ; i++ )
		{
			if ( TIMER1_msec_CountdownTimers[i] == COUNDOWNTIMER_UNUSE )
			{
				TIMER1_msec_CountdownTimers[i] = 0;
				return ( i ) ;
			}
		}	
		return(255);		
	}
	
/*******************************************************************************
 Setz einen Wert für den Countdowntimer von den an gezählt wird
*******************************************************************************/
void timer1_msec_SetCountdownTimer( unsigned char timer, unsigned int value )
	{
		cli();
		TIMER1_msec_CountdownTimers[ timer ] = value;
		sei();
	}
	
/*******************************************************************************
 Holt von einen Countdowntimer den aktuellen wert
*******************************************************************************/
unsigned int timer1_msec_GetCountdownTimer( unsigned char timer )
	{
		unsigned int t;
		cli();
		t = TIMER1_msec_CountdownTimers[ timer ];
		sei();
		return ( t );
	}
	
/*******************************************************************************
 Markiert den Timer als unbenutzt, nachdem er nicht mehr gebraucht wird
*******************************************************************************/
	void timer1_msec_ReleaseCountdownTimer( unsigned char timer )
	{
		cli();
		TIMER1_msec_CountdownTimers[ timer ] = COUNDOWNTIMER_UNUSE ;
		sei();
	}
//@}
